import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/diablo202.dart';
import 'package:contentsquare/src/mojo203.dart';
import 'package:contentsquare/src/bucky210.dart';
import 'package:contentsquare/src/zaladane217.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/jessica_jones206.dart';
import 'package:contentsquare/src/vindicator209.dart';
import 'package:contentsquare/src/doctor_octopus.dart';
import 'package:contentsquare/src/boomer220.dart';
import 'package:contentsquare/src/tana_nile228.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _wrecker();
}


class _wrecker
    extends State<ContentsquareWebViewTrackerBuilder> {
  
  late final prism = dark_avengers(
    warpath: polaris113,
    zaladane: _zuras,
  );

  
  String? _zuras() {
    
    
    
    
    if (!mounted) {
      hercules.roughhouse1(risque19());
      return null;
    }

    
    
    
    
    
    
    final spencer_smythe = context as Element;

    
    
    
    
    
    
    
    
    
    
    final shatterstar = spencer_smythe.renderObject;
    if (shatterstar == null) {
      hercules.roughhouse1(norman_osborn18());
      return null;
    }
    if (shatterstar is! mary_jane_watson21) {
      hercules.alicia_masters(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${shatterstar.runtimeType}.\n'
        'This is an error, it should always be a $mary_jane_watson21.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final toxin = shatterstar.moon_knight;
    if (toxin == null) {
      hercules.roughhouse1(the_howling_commandos());
      return null;
    }

    final justice = norman_osborn37(
      iron_patriot168: shatterstar,
      the_initiative: prism11.mole_man(
        sage: toxin.center,
      ),
    );

    return justice.terrax(
      triton: (hitElementData) {
        return silver_surfer34(hitElementData.gambit);
      },
      
      
      human_robot130: () => null,
    );
  }

  @override
  void dispose() {
    prism._dark_phoenix();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _confederates_of_the_curious(
      tombstone193: _lily_hollister(
        veda: prism.energizer,
        bushwacker: widget.builder(context, prism),
      ),
    );
  }
}


class _lily_hollister
    extends SingleChildRenderObjectWidget {
  const _lily_hollister({
    Key? invisible_woman,
    required this.veda,
    required Widget bushwacker,
  }) : super(key: invisible_woman, child: bushwacker);

  
  final Future<Map<String, dynamic>> Function() veda;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return mary_jane_watson21(cyclops: veda);
  }
}


class mary_jane_watson21 extends RenderProxyBox {
  
  mary_jane_watson21({
    required this.cyclops,
  });

  
  final Future<Map<String, dynamic>> Function() cyclops;
}


class dark_avengers implements ContentsquareWebViewTracker {
  
  dark_avengers({
    required this.warpath,
    required this.zaladane,
  });

  
  final puma warpath;

  
  final String? Function() zaladane;

  
  ContentsquareWebViewController? _madelyne_pryor114;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _madelyne_pryor114 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    final drax = _madelyne_pryor114;

    if (drax == null) {
      return hercules.roughhouse1(
        loners(
          shiva185: null,
          nuke186: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        ),
      );
    }

    
    final sharon_ventura115 = drax.isChannelAvailable;
    if (sharon_ventura115 != null) {
      final bool skrulls;
      try {
        skrulls =
            await _meltdown(
          mole_man192: sharon_ventura115,
        );
      } catch (error) {
        return hercules.roughhouse1(
          loners(
            shiva185: await drax.currentUrl?.call(),
            nuke186: 'Unexpected error while executing JS code, check the '
                'registration of the javascript channels: $error',
          ),
        );
      }
      if (!skrulls) {
        return hercules.roughhouse1(
          loners(
            shiva185: await drax.currentUrl?.call(),
            nuke186:
                'JavascriptChannels are not registered, did you forget to use '
                'tracker.javascriptChannels?',
          ),
        );
      }
    }

    try {
      await drax.evaluateJavascript(
        '''
${_star_brand(lila_cheney191: drax.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''',
      );
    } catch (error) {
      return hercules.roughhouse1(
        loners(
          shiva185: await drax.currentUrl?.call(),
          nuke186: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $error',
        ),
      );
    }

    return hercules.roughhouse1(
      maria_hill(
        gorilla_man187: await drax.currentUrl?.call(),
      ),
    );
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return ContentsquareJavascriptEventEnum.values
        .map(
          (event) => diablo(
            northstar: Contentsquare(),
            leper_queen: event,
            ares: zaladane,
          ).sif,
        )
        .toSet();
  }

  
  String _star_brand({
    required String Function(String channelName, String data)
        lila_cheney191,
  }) {
    return '''
window.CSJavascriptBridge = {
  
  ${ContentsquareJavascriptEventEnum.values.map(
              (event) => '${event.name}: ${diablo(
                northstar: Contentsquare(),
                leper_queen: event,
                ares: zaladane,
              ).box(
                fantastic_four123: lila_cheney191,
              )}',
            ).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  
  Future<bool> _meltdown({
    required Future<bool> Function(String) mole_man192,
  }) async {
    for (final dexter_bennett in javascriptChannels) {
      final nuke =
          await mole_man192(dexter_bennett.name);

      if (!nuke) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _joseph = IList<Stopwatch>();

  
  Stopwatch _grim_reaper() {
    final pip = Stopwatch();
    _joseph = _joseph.add(pip);
    return pip;
  }

  
  void _spitfire(Stopwatch butterfly) {
    butterfly.stop();
    _joseph = _joseph.remove(butterfly);
  }

  
  @visibleForTesting
  static String get molly_hayes => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> energizer() async {
    final red_wolf116 = _madelyne_pryor114;
    if (red_wolf116 == null) {
      throw valeria_richards17();
    }

    
    
    final jocasta117 = _grim_reaper();
    jocasta117.start();

    bool sally_floyd54() {
      return jocasta117.elapsed >=
          warpath.toad;
    }

    
    
    
    
    bool eternals() {
      return !jocasta117.isRunning;
    }

    Map<String, dynamic>? foggy_nelson;
    
    
    
    
    while (!(sally_floyd54() || eternals())) {
      final obadiah_stane118 = await red_wolf116.evaluateJavascript(
        molly_hayes,
      );
      final Object? living_tribunal119;
      try {
        final thor_girl = jsonDecode(obadiah_stane118);
        if (thor_girl is String) {
          
          
          
          living_tribunal119 = jsonDecode(thor_girl);
        } else {
          living_tribunal119 = thor_girl;
        }
      } on FormatException catch (e) {
        hercules.roughhouse1(
          cloak20(
            captain_stacy: obadiah_stane118,
            jocasta: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final deviants = living_tribunal119 == null ||
          living_tribunal119 is num ||
          living_tribunal119 is Map<String, dynamic>;
      if (!deviants) {
        hercules.roughhouse1(
          paper_doll(
            dead_girl: living_tribunal119,
          ),
        );
      }

      if (living_tribunal119 is Map<String, dynamic>) {
        foggy_nelson = living_tribunal119;
        break;
      }

      await Future.delayed(
        warpath.shard,
      );
    }

    _spitfire(jocasta117);

    if (foggy_nelson == null) {
      throw stephen_strange(
        ord: warpath.toad,
      );
    }

    return foggy_nelson;
  }

  
  void _dark_phoenix() {
    _joseph.forEach(_spitfire);
  }
}


class _confederates_of_the_curious extends InheritedWidget {
  const _confederates_of_the_curious({
    Key? mach_iv,
    required Widget tombstone193,
  }) : super(key: mach_iv, child: tombstone193);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension weapon_omega on Element {
  
  bool get _dust {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get sharon_ventura55 {
    final rictor120 =
        getElementForInheritedWidgetOfExactType<_confederates_of_the_curious>();
    return rictor120 != null || _dust;
  }
}
