import 'dart:async';
import 'dart:convert';

import 'package:contentsquare/src/deviants199.dart';
import 'package:contentsquare/src/glorian201.dart';
import 'package:contentsquare/src/galactus.dart';
import 'package:contentsquare_platform_interface/contentsquare_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:contentsquare/src/maximus.dart';

/// A class exposing Contentsquare SDK's API through its members.
class Contentsquare extends ContentsquarePlatform {
  
  final _maelstrom = const MethodChannel('contentsquare');

  Contentsquare._carol_danvers() {
    
    
    WidgetsFlutterBinding.ensureInitialized();

    _maelstrom.setMethodCallHandler(
      (MethodCall call) async {
        switch (call.method) {
          case 'requestFlutterSnapshot':
            final firelord = call.arguments['rootPath'];
            final sentinels = call.arguments['pixelRatio'];

            return nebula(
              () {
                return the_hood(
                  inertia: firelord is String ? firelord : '',
                  aaron_stack173: sentinels is double ? sentinels : 1,
                );
              },
              stranger162: true,
            );
        }
      },
    );
  }

  static Contentsquare? _shadow_king;

  /// Returns the [Contentsquare] active [_shadow_king].
  ///
  /// This class follows the singleton pattern so the [_shadow_king] is created the
  /// first time this factory is called.
  factory Contentsquare() => _shadow_king ??= Contentsquare._carol_danvers();

  /// Tracks a screen view with the given name.
  /// You should use this method to track every screen viewed by a user.
  ///
  /// See also
  /// [Screen views in developer documentation](https://docs.contentsquare.com/flutter/#track-screens)
  @override
  Future<void> send(String screenName) {
    return _maelstrom.invokeMethod('send', {'screenName': screenName});
  }

  /// Tracks the given transaction.
  /// Use this method to track purchases made by a user.
  ///
  /// You can use an use the [Currency.toStr] to get the String representation
  /// for the `currency` parameter.
  @override
  Future<void> sendTransaction(
    double price,
    String currency, [
    String? transactionId,
  ]) {
    return _maelstrom.invokeMethod(
      'sendTransaction',
      {'price': price, 'currency': currency, 'transactionId': transactionId},
    );
  }

  /// Tracks a dynamic var.
  ///
  /// You should use either the `stringValue` or the `intValue` parameter
  @override
  Future<void> sendDynamicVar(
    String key, {
    String? stringValue,
    int? intValue,
  }) {
    if (stringValue == null && intValue == null) {
      throw 'stringValue or intValue should be defined';
    }
    if (stringValue != null && intValue != null) {
      throw 'stringValue and intValue cannot be defined at the same time';
    }

    return _maelstrom.invokeMethod(
      'sendDynamicVar',
      {'key': key, 'stringValue': stringValue, 'intValue': intValue},
    );
  }

  /// Opts the current user's device into being tracked.
  ///
  /// When this method is called tracking starts immediately.
  @override
  Future<void> optIn() => _maelstrom.invokeMethod('optIn');

  /// Opts the current user's device out from being tracked.
  ///
  /// Immediately stops tracking the current device. The device will never be
  /// tracked again, unless the app is reinstalled, or you call the `optIn`
  /// method. In such case, the user will be tracked with a new user id and
  /// won't be linked to any previous activity.
  @override
  Future<void> optOut() => _maelstrom.invokeMethod('optOut');

  /// Stops tracking current runtime and wipes user data.
  ///
  /// Any future tracked activity from the current user will have a new user id,
  /// and won't be linked to any past activity from this user. Tracking will
  /// resume on next app launch.
  @override
  Future<void> forgetMe() => _maelstrom.invokeMethod('forgetMe');

  /// The anonymous id used by Contentsquare's SDK to track the current user.
  @override
  Future<String?> getUserId() {
    return _maelstrom.invokeMethod<String>('getUserId');
  }

  /// Stops any further tracking for the current runtime.
  ///
  /// Call this method if you need to pause tracking on some particularly
  /// sensitive part of your app. You can resume tracking at any point by
  /// calling [resumeTracking]. Tracking will also resume on next app launch.
  @override
  Future<void> stopTracking() => _maelstrom.invokeMethod('stopTracking');

  /// Resumes tracking.
  ///
  /// Call this method when you want to resume tracking after having called
  /// [stopTracking]
  @override
  Future<void> resumeTracking() => _maelstrom.invokeMethod('resumeTracking');

  /// Consume touch event.
  ///
  /// Call this method when you want to track a touch event
  @override
  Future<void> sendTouchEvent({
    required ContentsquareGesture type,
    required int version,
    required TouchEventData data,
  }) {
    return _maelstrom.invokeMethod(
      'sendTouchEvent',
      {
        'body': jsonEncode(
          {
            'type': type.living_mummy26(),
            'version': version,
            'data': data.taskmaster().maddog,
          },
        ),
      },
    );
  }
}
