import 'package:contentsquare/src/modules/logger/logger.dart';
import 'package:flutter_test/flutter_test.dart';

/// A method to easily test the logs emitted by the [ContentsquareLogger].
///
/// [onLog] is a callback that will be called each time a log is emitted.
///
/// [timeOutDuration] is the maximum duration of the test, if exceeded, the
/// test will fail. Defaults to 30 second.
///
/// [count] is the number of logs that should be emitted. Defaults to 1.
///
/// WARNING: There is a bug on AndroidStudio that causes the test to hang
/// without timing out if [count] is never reached. Therefore on AndroidStudio,
/// do always run your tests from the terminal.
/// Link to the issue: https://github.com/flutter/flutter/issues/97825
void expectLogs(
  void Function(ContentsquareLog log) onLog, {
  Duration timeOutDuration = const Duration(seconds: 30),
  int count = 1,
}) {
  // Scope the logger to prevent it from being shared with other tests.
  logger.overrideForTesting(
    onLog: expectAsync1<void, ContentsquareLog>(onLog, count: count),
    addTearDown: addTearDown,
  );
}

/// Wrapper around [expectLogs] that allows to test that no logs have been
/// emitted.
void expectNoLogs() => expectLogs((_) {}, count: 0);
