import 'package:flutter/material.dart';
import 'package:test/fake.dart';
import 'package:webview_flutter/webview_flutter.dart';

class FakeWebView extends StatefulWidget with Fake implements WebView {
  const FakeWebView({
    Key? key,
    this.javascriptMode = JavascriptMode.disabled,
    this.onPageFinished,
    this.onWebViewCreated,
    required this.onRunJavascriptReturningResult,
  }) : super(key: key);

  @override
  final JavascriptMode javascriptMode;

  @override
  final PageFinishedCallback? onPageFinished;

  @override
  final WebViewCreatedCallback? onWebViewCreated;

  final Future<String> Function(String javascript)
      onRunJavascriptReturningResult;

  @override
  State<StatefulWidget> createState() => _FakeWebViewState();
}

class _FakeWebViewState extends State<FakeWebView> {
  late final WebViewController _controller = FakeWebViewController(
    onRunJavascriptReturningResult: (javascript) {
      if (widget.javascriptMode == JavascriptMode.unrestricted) {
        return widget.onRunJavascriptReturningResult(javascript);
      } else {
        return throw 'Cannot run javascript in ${widget.javascriptMode}';
      }
    },
  );

  @override
  void initState() {
    widget.onWebViewCreated?.call(_controller);
    widget.onPageFinished?.call('https://fake.com');
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return const ColoredBox(
      color: Colors.red,
      child: Center(
        child: Text('Fake WebView'),
      ),
    );
  }
}

class FakeWebViewController extends Fake implements WebViewController {
  FakeWebViewController({
    required this.onRunJavascriptReturningResult,
  });

  final Future<String> Function(String javascriptString)
      onRunJavascriptReturningResult;

  @override
  Future<String?> currentUrl() async => 'https://fake.com';

  @override
  Future<String> runJavascriptReturningResult(String javascriptString) {
    return onRunJavascriptReturningResult(javascriptString);
  }
}
