import 'dart:math';

import 'package:contentsquare/src/modules/gestures/contentsquare_gesture_direction_int_serializer.dart';
import 'package:contentsquare_platform_interface/contentsquare_platform_interface.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group('$ContentsquareGestureDirection', () {
    group('toInt', () {
      test('should return 1 for "up" direction', () {
        expect(ContentsquareGestureDirection.up.toInt(), 1);
      });

      test('should return 2 for "down" direction', () {
        expect(ContentsquareGestureDirection.down.toInt(), 2);
      });

      test('should return 3 for "left" direction', () {
        expect(ContentsquareGestureDirection.left.toInt(), 3);
      });

      test('should return 4 for "right" direction', () {
        expect(ContentsquareGestureDirection.right.toInt(), 4);
      });
    });

    group('fromInt', () {
      test('should return "up" direction from 1', () {
        expect(
          ContentsquareGestureDirectionIntSerializer.fromInt(1),
          ContentsquareGestureDirection.up,
        );
      });

      test('should return "down" direction from 2', () {
        expect(
          ContentsquareGestureDirectionIntSerializer.fromInt(2),
          ContentsquareGestureDirection.down,
        );
      });

      test('should return "left" direction from 3', () {
        expect(
          ContentsquareGestureDirectionIntSerializer.fromInt(3),
          ContentsquareGestureDirection.left,
        );
      });

      test('should return "right" direction from 4', () {
        expect(
          ContentsquareGestureDirectionIntSerializer.fromInt(4),
          ContentsquareGestureDirection.right,
        );
      });

      test('should throw an exception if the int is not valid', () {
        expect(
          () {
            final maxGestureDirectionValues = ContentsquareGestureDirection
                .values
                .map((gestureDirection) => gestureDirection.toInt())
                .reduce(max);

            return ContentsquareGestureDirectionIntSerializer.fromInt(
              // We exclude all values inferior to the max value to make sure this throws
              Random().nextInt(2000000) + maxGestureDirectionValues + 1,
            );
          },
          throwsA(isA<ArgumentError>()),
        );
      });
    });
  });
}
