import 'dart:math';

import 'package:contentsquare/src/modules/gestures/contentsquare_gesture_int_serializer.dart';
import 'package:contentsquare_platform_interface/contentsquare_platform_interface.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group('ContentsquareGestureIntSerializer', () {
    group('toInt', () {
      test('should return 4 for "page view" gesture', () {
        expect(ContentsquareGesture.pageView.toInt(), 4);
      });

      test('should return 6 for "tap" gesture', () {
        expect(ContentsquareGesture.tap.toInt(), 6);
      });

      test('should return 6 for "long press" gesture', () {
        expect(ContentsquareGesture.longPress.toInt(), 8);
      });

      test('should return 6 for "drag" gesture', () {
        expect(ContentsquareGesture.drag.toInt(), 9);
      });

      test('should return 6 for "flick" gesture', () {
        expect(ContentsquareGesture.flick.toInt(), 10);
      });
    });

    group('fromInt', () {
      test('should return "page view" gesture from 4', () {
        expect(
          ContentsquareGestureIntSerializer.fromInt(4),
          ContentsquareGesture.pageView,
        );
      });

      test('should return "tap" gesture from 6', () {
        expect(
          ContentsquareGestureIntSerializer.fromInt(6),
          ContentsquareGesture.tap,
        );
      });

      test('should return "long press" gesture from 8', () {
        expect(
          ContentsquareGestureIntSerializer.fromInt(8),
          ContentsquareGesture.longPress,
        );
      });

      test('should return "drag" gesture from 9', () {
        expect(
          ContentsquareGestureIntSerializer.fromInt(9),
          ContentsquareGesture.drag,
        );
      });

      test('should return "flick" gesture from 10', () {
        expect(
          ContentsquareGestureIntSerializer.fromInt(10),
          ContentsquareGesture.flick,
        );
      });

      test('should throw an exception if the int is not valid', () {
        expect(
          () {
            final maxGestureValue = ContentsquareGesture.values
                .map((gesture) => gesture.toInt())
                .reduce(max);

            return ContentsquareGestureIntSerializer.fromInt(
              // We exclude all values inferior to the max value to make sure this throws
              Random().nextInt(2000000) + maxGestureValue + 1,
            );
          },
          throwsA(isA<ArgumentError>()),
        );
      });
    });
  });
}
