import 'package:contentsquare/src/modules/logger/logger.dart';
import 'package:contentsquare/src/modules/logger/run_in_error_logger_zone.dart';
import 'package:flutter_test/flutter_test.dart';

import '../../../helpers/expect_logs.dart';

void main() {
  group(
    '$runInErrorLoggerZone',
    () {
      test(
        'should return the result of the given callback '
        'when no error is thrown',
        () {
          int callback() => 3;

          expect(runInErrorLoggerZone(callback), equals(3));
        },
      );

      test(
        'should log the error and return null '
        'when the callback throws and rethrowError is false',
        () {
          int callback() => throw StateError('');
          expectLogs((log) {
            expect(log, isA<ErrorLog>());
          });

          expect(
            runInErrorLoggerZone(callback, rethrowErrors: false),
            equals(null),
          );
        },
      );

      test(
        'should log the error and rethrow '
        'when callback throws and rethrowError is true',
        () {
          int callback() => throw StateError('');
          expectLogs((log) {
            expect(log, isA<ErrorLog>());
          });

          expect(
            () => runInErrorLoggerZone(callback, rethrowErrors: true),
            throwsA(isA<StateError>()),
          );
        },
      );
    },
  );
}
