import 'package:contentsquare/src/modules/webview_tracking/exceptions/screengraph_retrieval_time_out_exception.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group(
    '$ScreengraphRetrievalTimeOutException',
    () {
      const customDuration = Duration(seconds: 5);

      ScreengraphRetrievalTimeOutException getException({
        Duration maxDuration = const Duration(seconds: 1),
      }) =>
          ScreengraphRetrievalTimeOutException(maxDuration: maxDuration);

      test(
        'should be an instance of the $Exception',
        () {
          final exception = getException();

          expect(exception, isA<Exception>());
        },
      );

      test(
        'should have same internal param when created with custom maxDuration',
        () {
          final exception = getException(maxDuration: customDuration);

          expect(exception.maxDuration, equals(customDuration));
        },
      );

      test(
        'should have expected toString() message '
        'when created with custom maxDuration',
        () {
          final exception = getException(maxDuration: customDuration);
          final expected = 'Timed out after trying to get screengraph for '
              '${customDuration.inMilliseconds}ms unsuccessfully.';

          expect(exception.toString(), equals(expected));
        },
      );
    },
  );
}
