import 'dart:math';

import 'package:contentsquare/src/modules/logger/logger.dart';
import 'package:contentsquare/src/modules/webview_tracking/logs/unknown_javascript_screengraph_format_log.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group(
    '$UnknownJavascriptScreengraphFormatLog',
    () {
      const customFormatException = FormatException('custom message');
      final customObject = Random();

      UnknownJavascriptScreengraphFormatLog getErrorLog({
        FormatException jsonDecodeError = const FormatException(),
        Object? screengraphOfUnknownType = '',
      }) {
        return UnknownJavascriptScreengraphFormatLog(
          screengraphOfUnknownType: screengraphOfUnknownType,
          jsonDecodeError: jsonDecodeError,
        );
      }

      test(
        'should be an instance of the $ErrorLog',
        () {
          final exception = getErrorLog();

          expect(exception, isA<ErrorLog>());
        },
      );

      test(
        'should screengraphOfUnknownType be a nullable',
        () {
          final exception = getErrorLog(screengraphOfUnknownType: null);

          expect(exception.screengraphOfUnknownType, isNull);
          expect(exception.screengraphOfUnknownType, isA<Object?>());
        },
      );

      test(
        'should have same internal params '
        'when created instance with custom params',
        () {
          final instance = getErrorLog(
            jsonDecodeError: customFormatException,
            screengraphOfUnknownType: customObject,
          );

          expect(instance.jsonDecodeError, equals(customFormatException));
          expect(instance.screengraphOfUnknownType, equals(customObject));
        },
      );

      test(
        'should have expected message param',
        () {
          final instance = getErrorLog(
            jsonDecodeError: customFormatException,
            screengraphOfUnknownType: customObject,
          );
          final expectedMessage =
              'Unknown Javascript screengraph format. Expected: '
              'Map<String, dynamic> or null.\n'
              'jsonDecode error: $customFormatException\n'
              'Screengraph: $customObject';
          expect(instance.message, equals(expectedMessage));
        },
      );
    },
  );
}
