import 'dart:math';

import 'package:contentsquare/src/modules/logger/logger.dart';
import 'package:contentsquare/src/modules/webview_tracking/logs/unknown_javascript_screengraph_type_log.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group(
    '$UnknownJavascriptScreengraphTypeLog',
    () {
      final customObject = Random();

      UnknownJavascriptScreengraphTypeLog getErrorLog({
        Object? screengraphOfUnknownType = 42,
      }) {
        return UnknownJavascriptScreengraphTypeLog(
          screengraphOfUnknownType: screengraphOfUnknownType,
        );
      }

      test(
        'should be an instance of the $ErrorLog',
        () {
          final exception = getErrorLog();

          expect(exception, isA<ErrorLog>());
        },
      );

      test(
        'should screengraphOfUnknownType param be a nullable',
        () {
          final exception = getErrorLog(screengraphOfUnknownType: null);

          expect(exception.screengraphOfUnknownType, isNull);
          expect(exception.screengraphOfUnknownType, isA<Object?>());
        },
      );

      test(
        'should have same internal params '
        'when created instance with custom  params',
        () {
          final instance = getErrorLog(screengraphOfUnknownType: customObject);

          expect(instance.screengraphOfUnknownType, equals(customObject));
        },
      );

      test(
        'should have expected message param',
        () {
          final instance = getErrorLog(screengraphOfUnknownType: customObject);
          final expectedMessage =
              'Unknown Javascript screengraph type: ${customObject.runtimeType}'
              '. Expected: Map<String, dynamic> or null.\n'
              'Screengraph: $customObject';
          expect(instance.message, equals(expectedMessage));
        },
      );
    },
  );
}
