import 'package:contentsquare/src/modules/logger/logger.dart';
import 'package:contentsquare/src/modules/webview_tracking/javascript_event_handler.dart';
import 'package:contentsquare/src/modules/webview_tracking/logs/logs.dart';
import 'package:contentsquare/src/modules/webview_tracking/logs/webview_event_log.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group(
    '$WebViewEventLog',
    () {
      const customEvent = ContentsquareJavascriptEventEnum.optOut;
      const customMessage = 'Hello there';

      WebViewEventLog getEventLog({
        ContentsquareJavascriptEventEnum event =
            ContentsquareJavascriptEventEnum.optIn,
        String message = 'not empty',
      }) {
        return _TestWebViewEventLog(
          event: event,
          message: message,
        );
      }

      test(
        'should be an instance of the $ContentsquareLog and $WebViewLogPrefix',
        () {
          final exception = getEventLog();

          expect(exception, isA<ContentsquareLog>());
          expect(exception, isA<WebViewLogPrefix>());
        },
      );

      test(
        'should have same internal params when created instance with custom '
        'params',
        () {
          final instance = getEventLog(
            event: customEvent,
            message: customMessage,
          );
          const expectedHeader = '(CSLIB) [Tracked WebView] ⟶ Event "optOut"';

          expect(instance.event, equals(customEvent));
          expect(instance.message, equals(customMessage));
          expect(instance.header, equals(expectedHeader));
        },
      );
    },
  );
}

class _TestWebViewEventLog extends WebViewEventLog {
  _TestWebViewEventLog({
    required ContentsquareJavascriptEventEnum event,
    required String message,
  }) : super(event: event, message: message);
}
