import 'dart:collection';

import 'package:contentsquare/src/shared/json/json_object.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group(
    '$JsonObject',
    () {
      test(
        'should return normally '
        'when passing an empty map',
        () {
          expect(
            () => JsonObject(HashMap()),
            returnsNormally,
          );
        },
      );

      test(
        'should return normally '
        'when passing correct json map',
        () {
          expect(
            () => JsonObject({'key': 'value'}),
            returnsNormally,
          );
        },
      );

      test(
        'should return true '
        'when compare value from $JsonObject and $Map with the same values',
        () {
          const key = 'key';
          const value = 'value';
          final jsonObject = JsonObject({key: value});
          final map = {key: value};

          expect(
            jsonObject.encodable,
            map,
          );
        },
      );

      test(
        'should throw '
        'when passing object that could NOT be encoded to json',
        () {
          expect(
            () => JsonObject({
              'key': const IListConst(['value'])
            }),
            throwsAssertionError,
          );
        },
      );
    },
  );
}
